@@ -39,7 +39,7 @@
 #define UF_IMMUTABLE 0
 #endif
 
-#define	INOHSZ	512
+#define	INOHSZ	8192
 #if	((INOHSZ&(INOHSZ-1)) == 0)
 #define	INOHASH(ino)	((ino)&(INOHSZ-1))
 #else
@@ -47,6 +47,7 @@
 #endif
 
 static struct node *nodehash[INOHSZ];
+static size_t nodehash_nr_items;
 
 static error_t read_node (struct node *np);
 
@@ -106,6 +107,7 @@ diskfs_cached_lookup (ino_t inum, struct node **npp)
     dn->hnext->dn->hprevp = &dn->hnext;
   dn->hprevp = &nodehash[INOHASH(inum)];
   nodehash[INOHASH(inum)] = np;
+  nodehash_nr_items += 1;
 
   pthread_spin_unlock (&diskfs_node_refcnt_lock);
 
@@ -159,6 +161,7 @@ diskfs_node_norefs (struct node *np)
   *np->dn->hprevp = np->dn->hnext;
   if (np->dn->hnext)
     np->dn->hnext->dn->hprevp = np->dn->hprevp;
+  nodehash_nr_items -= 1;
 
   if (np->dn->dirents)
     free (np->dn->dirents);
@@ -549,7 +552,8 @@ error_t
 diskfs_node_iterate (error_t (*fun)(struct node *))
 {
   error_t err = 0;
-  int n, num_nodes = 0;
+  int n;
+  size_t num_nodes;
   struct node *node, **node_list, **p;
 
   pthread_spin_lock (&diskfs_node_refcnt_lock);
@@ -559,10 +563,7 @@ diskfs_node_iterate (error_t (*fun)(struct node *))
      during processing (normally we delegate access to hash-table with
      diskfs_node_refcnt_lock, but we can't hold this while locking the
      individual node locks).  */
-
-  for (n = 0; n < INOHSZ; n++)
-    for (node = nodehash[n]; node; node = node->dn->hnext)
-      num_nodes++;
+  num_nodes = nodehash_nr_items;
 
   /* TODO This method doesn't scale beyond a few dozen nodes and should be
      replaced.  */
@@ -732,7 +733,7 @@ diskfs_set_translator (struct node *np, const char *name, unsigned namelen,
 
       buf[0] = namelen & 0xFF;
       buf[1] = (namelen >> 8) & 0xFF;
-      bcopy (name, buf + 2, namelen);
+      memcpy (buf + 2, name, namelen);
 
       blkptr = disk_cache_block_ref (blkno);
       memcpy (blkptr, buf, block_size);
@@ -804,7 +805,7 @@ write_symlink (struct node *node, const char *target)
 
   assert (node->dn_stat.st_blocks == 0);
 
-  bcopy (target, node->dn->info.i_data, len);
+  memcpy (node->dn->info.i_data, target, len);
   node->dn_stat.st_size = len - 1;
   node->dn_set_ctime = 1;
   node->dn_set_mtime = 1;
@@ -821,7 +822,7 @@ read_symlink (struct node *node, char *target)
 
   assert (node->dn_stat.st_size < MAX_INODE_SYMLINK);
 
-  bcopy (node->dn->info.i_data, target, node->dn_stat.st_size);
+  memcpy (target, node->dn->info.i_data, node->dn_stat.st_size);
   return 0;
 }
 
