@@ -27,17 +27,32 @@ _ports_complete_deallocate (struct port_info *pi)
 {
   assert ((pi->flags & PORT_HAS_SENDRIGHTS) == 0);
 
-  if (pi->port_right)
+  if (MACH_PORT_VALID (pi->port_right))
     {
+      struct references result;
+
+      pthread_rwlock_wrlock (&_ports_htable_lock);
+      refcounts_references (&pi->refcounts, &result);
+      if (result.hard > 0 || result.weak > 0)
+        {
+          /* A reference was reacquired through a hash table lookup.
+             It's fine, we didn't touch anything yet. */
+          /* XXX: This really shouldn't happen.  */
+          assert (! "reacquired reference w/o send rights");
+          pthread_rwlock_unlock (&_ports_htable_lock);
+          return;
+        }
+
+      hurd_ihash_locp_remove (&_ports_htable, pi->ports_htable_entry);
       hurd_ihash_locp_remove (&pi->bucket->htable, pi->hentry);
+      pthread_rwlock_unlock (&_ports_htable_lock);
+
       mach_port_mod_refs (mach_task_self (), pi->port_right,
 			  MACH_PORT_RIGHT_RECEIVE, -1);
       pi->port_right = MACH_PORT_NULL;
     }
 
-  *pi->prevp = pi->next;
-  if (pi->next)
-    pi->next->prevp = pi->prevp;
+  pthread_mutex_lock (&_ports_lock);
 
   pi->bucket->count--;
   pi->class->count--;
@@ -49,4 +64,3 @@ _ports_complete_deallocate (struct port_info *pi)
   
   free (pi);
 }
-
