@@ -34,6 +34,9 @@
 #include <hurd/trivfs.h>
 #include <version.h>
 
+#include "libtrivfs/trivfs_fs_S.h"
+#include "libtrivfs/trivfs_io_S.h"
+
 /* The global lock */
 pthread_mutex_t global_lock;
 
@@ -625,6 +628,8 @@ trivfs_S_file_set_size (struct trivfs_protid *cred,
     return EOPNOTSUPP;
   else if (!(cred->po->openmodes & O_WRITE))
     return EBADF;
+  else if (size < 0)
+    return EINVAL;
   else
     return 0;
 }
